#include "DirectX2dPainter.h"

#include "DirectX2dInterface.h"
#include "AbstractGeometricItem.h"
#include "Rectangle.h"
#include "SceneModel.h"

void DirectX2dPainter::startDrawing()
{
	mD2dInterface->getRenderTarget()->BeginDraw();
}

void DirectX2dPainter::finishDrawing()
{
	mD2dInterface->getRenderTarget()->EndDraw();
}

void DirectX2dPainter::clearBackground()
{
	mD2dInterface->getRenderTarget()->Clear(D2D1::ColorF(D2D1::ColorF::White));
}

void DirectX2dPainter::paint(SceneModel* model)
{
	auto rt = mD2dInterface->getRenderTarget();

	auto hr = rt->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black), &mWorkingBrush);

	if (model->getGeometry()->getType() == AbstractGeometricItem::Type::RECTANGLE)
	{
		auto rect = dynamic_cast<ntk::Rectangle*>(model->getGeometry());
		const auto loc = rect->getLocation();
		const auto width = rect->getWidth();
		const auto height = rect->getHeight();

		D2D1_RECT_F d2d_rect{ static_cast<float>(loc.getX()), static_cast<float>(loc.getY() + height), static_cast<float>(loc.getX() + width), static_cast<float>(loc.getY()) };

		rt->FillRectangle(d2d_rect, mWorkingBrush.Get());
	}
}

void DirectX2dPainter::setD2dInterface(DirectX2dInterface* d2dIterface)
{
	mD2dInterface = d2dIterface;
}